/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include "creachgraph.h"
          
//--------------------------------------------------------------------------
/** Creates a CReachGraph with data taken from \a cnet. */

CReachGraph::CReachGraph(CNet *cnet)
{
  this->cnet = cnet;
  
  placeNumber = cnet->placesCount();
  transNumber = cnet->transitionsCount();
  finished = true;
  if(placeNumber == 0 || transNumber == 0)
  {
    empty = true;
    return;
  }
  empty = false;
  
  inM = new int*[placeNumber];
  outM = new int*[placeNumber];
  initMarking = new int[placeNumber];
  priority = new int[transNumber];
  time = new int[transNumber];
  
  for(int i = 0; i < placeNumber; i++)
  {
    inM[i] = new int[transNumber];
    outM[i] = new int[transNumber] ;
  }
  
  maxStNumber = 100; //DEFAULT_MAX_STATE_NUMBER;
  getInMatrix();
  getOutMatrix();
  getPriorities();
  getTimes();
  getInitialMarking();
}


//--------------------------------------------------------------------------
/** Destroys the CReachGraph. */

CReachGraph::~CReachGraph()
{
  if(!empty)
  {
	  delete initMarking;
	  delete priority;	
	  delete time;
  
	  for(int i = 0; i < placeNumber; i++)
    {
	    delete inM[i];
	    delete outM[i];
	  }
	  delete inM;
	  delete outM;

	clearList();
  }
}


//--------------------------------------------------------------------------
/** Deletes all states in states' list. */

void CReachGraph::clearList()
{
  states.setAutoDelete(true);
  states.clear();
  states.setAutoDelete(false);  
}


//--------------------------------------------------------------------------
/** Collects data for the input matrix. */

void CReachGraph::getInMatrix()
{
  int **tmpM = cnet->inMatrix();
  int i, j;
  
  if(tmpM == NULL) exit(1);
    
  for(i = 0; i < placeNumber; i++)
  {
    for(j = 0; j < transNumber; j++)
    {
      inM[i][j] = tmpM[i][j];
    }
  }
}


//--------------------------------------------------------------------------
/** Collects data for the output matrix. */

void CReachGraph::getOutMatrix()
{
  int **tmpM = cnet->outMatrix();
  int i, j;
  
  if(tmpM == NULL) exit(1);
  
  for(i = 0; i < placeNumber; i++)
  {
    for(j = 0; j < transNumber; j++)
    {
      outM[i][j] = tmpM[i][j];
    }
  }
}


//--------------------------------------------------------------------------
/** Collects data for the transition priorities. */

void CReachGraph::getPriorities()
{
  for(int i = 0; i < transNumber; i++)
  {
    priority[i] = cnet->transition(i)->priority();
  }
}


//--------------------------------------------------------------------------
/** Collects data for the transition times. */

void CReachGraph::getTimes()
{
  for(int i = 0; i < transNumber; i++)
  {
    time[i] = cnet->transition(i)->time();
  }
}


//--------------------------------------------------------------------------
/** Collects data for the initial marking. */

void CReachGraph::getInitialMarking()
{
  for(int i = 0; i < placeNumber; i++)
  {
    initMarking[i] = cnet->place(i)->initialMarking();
  }		
}


//--------------------------------------------------------------------------
/** Sets the maximum number of states in the reachability 
  graph that will be computed. */

void CReachGraph::setMaxStateNumber(int nr)
{
  if(nr > 0) maxStNumber = nr;
}



//--------------------------------------------------------------------------
/** Creates reachability graph. */

void CReachGraph::createGraph()
{
  if(empty) return;
  
  finished = false;	 
  
  if(withTime)
  {
    createGraphWithTime();
  } 
  else 
  {
    createGraphWithoutTime();
  }
}


//--------------------------------------------------------------------------
/** Returns the raechability graph description. */

QString CReachGraph::graphDescription()
{
  QString info("");
  
  for(cs = states.first();  cs != NULL; cs= states.next())
  {
    if(withTime)
    {
      info += cs->info(true);
      info += cs->successors(cnet, true) + "\n";
    }
    else
    {
      info += cs->info(false);
      info += cs->successors(cnet, false) + "\n";
    }
  }
  if(!finished)
  {
    info += "...(there is more states)\n";
  }
  return info;
}


//--------------------------------------------------------------------------
/** Generates the reachability graph (time is not used). */

void CReachGraph::createGraphWithoutTime()
{
  CState* curState = new CState(0, placeNumber, transNumber);
  int i, j, k;
  
  for(i = 0; i < placeNumber; i++)
  {
    curState->setMarking(i, initMarking[i]);
  } 
  
  // find active transitions
  int* active  = new int[transNumber];
  int* nactive = new int[transNumber];
  int* delta   = new int[transNumber];
  
  for(i = 0; i < transNumber; i++)
  {
    active[i] = 1;
    for(j = 0; j < placeNumber; j++)
    {
      if(curState->marking(j) < outM[j][i])
      {
        active[i] = 0;
        break;
      }
    }

    if(active[i] > 0)
    {
      curState->setDelta(i,0);
    }
    else
    {
      curState->setDelta(i,-1);
    }
  }   
 
  // append the initial state to the list 
  states.append(curState);
  stateIndex = 0;   // number of the current state
  int index;
  
  // generate the reachability graph
  
  QPtrListIterator<CState> it(states);   
  
  while((cs = it.current()) != NULL && stateIndex < maxStNumber)
  {   
    curState = cs;
    bool found = false;
    
    // copy enable transitions    
    for(i = 0; i < transNumber; i++)
    {
      active[i] = curState->delta(i);
      // check whether there exists at least one enable transition
      if(active[i] >= 0)
      { 
        found=true;
      }
    }
      
    if(withPriorities == true) // check priorities  
    {
      for(i = 0; i < transNumber - 1; i++)
      {
        for(j = i + 1; j < transNumber; j++) 
        {
          if((active[i] >= 0) && (active[j] >= 0) 
            && (priority[i] != priority[j]))
          {
            for(k = 0; k < placeNumber; k++)
            {
              if((outM[k][i] > 0) && (outM[k][j] > 0) 
                && (outM[k][i] + outM[k][j] > curState->marking(k)))
              {
                if(priority[i] > priority[j])
                {
                  active[j] = -1;
                }
                else
                {
                  active[i] = -1;
                }
              }
            }
          }
        }
      } 
    }
      
    // generate the new states 
    index = 0;
    while(found && (index < transNumber)) 
    {
      if(active[index] >= 0) 
      {
        CState* newState = new CState(-1, placeNumber, transNumber); // new state
        int k;
        
        // new marking
        for(i = 0; i < placeNumber; i++)
        {
          k = curState->marking(i);
          k = k - outM[i][index] + inM[i][index];
          newState->setMarking(i,k);
        }
        
        // new dynamic delays vector
        for(i = 0; i < transNumber; i++)
        {
          nactive[i] = 1;
          for(j = 0; j < placeNumber; j++)
          {
            if(newState->marking(j) < outM[j][i])
            {
              nactive[i]= 0;
              break;
            }
          }
          if(nactive[i] == 1)
          {
            newState->setDelta(i,0);
          }
          else
          {
            newState->setDelta(i,-1);
          }
        }
      
        // check whether such a state already exists
        QPtrListIterator<CState> it2(states);
        CState* cs2;
        bool inList = false;
        int findState = -1;
        for(; (cs2 = it2.current()) != NULL; ++it2)
        {
          if(cs2->equal(newState)) 
          {
            inList = true;
            findState = cs2->tag();
          }
        }
        
        // add new state to the list
        if(inList == false) 
        {
          newState->setTag(++stateIndex);
          curState->addSucc(index, stateIndex); // add successor
          states.append(newState);
        }
        else
        {
          curState->addSucc(index, findState); // add successor
          delete newState;
        }
        
      } // end if (active[index] >= 0)
        
      index++;    
    } // end while(found && index < transNumber)
      
    ++it;
  } // end while((cs = it.current()) != NULL && stateIndex < maxStNumber)
  
  if(cs == NULL)
  {
    finished = true;
  }

  delete active;
  delete nactive;
  delete delta;
}


//--------------------------------------------------------------------------
/** Generates the reachability graph (time is used). */

void CReachGraph::createGraphWithTime()
{
    CState* curState = new CState(0, placeNumber, transNumber);
  int i, j, k;
  
  for(i = 0; i < placeNumber; i++)
  {
    curState->setMarking(i, initMarking[i]);
  } 
  
  // find active transitions
  int* active  = new int[transNumber];
  int* nactive = new int[transNumber];
  int* delta   = new int[transNumber];
    
  for(i = 0; i < transNumber; i++)
  {
    active[i] = 1;
    for(j = 0; j < placeNumber; j++)
    {
      if(curState->marking(j) < outM[j][i])
      {
        active[i] = 0;
        break;
      }
    }

    if(active[i] > 0)
    {
      curState->setDelta(i,time[i]);
    }
    else
    {
      curState->setDelta(i,-1);
    }
  }   
  
  // append the initial state to the list
  states.append(curState);
  stateIndex = 0;   
  int index;  
  int minTime = -1;
  
  // generate the reachability graph
  
  QPtrListIterator<CState> it(states);
  
  while((cs = it.current()) != NULL && stateIndex < maxStNumber)
  {   
    curState = cs;
    bool found = false;
    minTime = -1;

    // copy enable transitions    
    for(i = 0; i < transNumber; i++)
    {
      active[i] = curState->delta(i);
      // find the minimal delay time
      if(active[i] >= 0) 
      { 
        found = true;
        if(minTime < 0)
        {
          minTime = active[i];
        }
        else
        {
          if(minTime >= active[i])
          {
            minTime = active[i];
          }
        }    
      }
    }  
    
    // check priorities  
    if(withPriorities == true)
    {
      for(i = 0; i < transNumber - 1; i++)
      {
        for(j = i + 1; j < transNumber; j++) 
        {
          if((active[i] >= 0) && (active[j] >= 0) 
            && (priority[i] != priority[j]))
          {
            for(k = 0; k < placeNumber; k++)
            {
              if((outM[k][i] > 0) && (outM[k][j] > 0) 
                && (outM[k][i] + outM[k][j] > curState->marking(k)))
              {
                if(priority[i] > priority[j])
                {
                  active[j] = -1;
                }
                else
                {
                  active[i] = -1;
                }
              }
            }
          }
        }
      }
    }
      
    // generate the new states 
    index = 0;
    while(found && (index < transNumber)) 
    {
      if(active[index] == minTime )
      {     
        CState* newState = new CState(-1, placeNumber, transNumber); // new state
        int k;
        
        // new marking
        for(i = 0; i < placeNumber; i++)
        {
          k = curState->marking(i);
          k = k - outM[i][index] + inM[i][index];
          newState->setMarking(i,k);
        }
          
        // new dynamic delays vector
        for(i = 0; i < transNumber; i++)
        {
          nactive[i] = 1;
          for(j = 0; j < placeNumber; j++)
          {
            if(newState->marking(j) < outM[j][i])
            {
              nactive[i]= 0;
              break;
            }
          }         
        }
            
        for(i = 0; i < transNumber; i++)
        {
          if(nactive[i] > 0)
          {
            if(active[i] >= 0)
            {
              if(i != index)
              {
                newState->setDelta(i,active[i]-minTime);
              }
              else
              {
                newState->setDelta(i,time[i]);
              }
            }
            else
            {
              newState->setDelta(i,time[i]);
            }
          }
          else
          {
            newState->setDelta(i,-1);
          }
        }
        
        // check whether such a state already exists
        QPtrListIterator<CState> it2(states);
        CState* cs2;
        bool inList = false;
        int findState = -1;
        for(; (cs2 = it2.current()) != NULL; ++it2)
        {
          if(cs2->equal(newState)) 
          {
            inList = true;
            findState = cs2->tag();
          }
        }
      
        // add the new state to the list
        if(inList == false) 
        {
          newState->setTag(++stateIndex);
          curState->addSucc(index, stateIndex); // add successor
          states.append(newState);
        }
        else
        {
          curState->addSucc(index, findState); // add successor
          delete newState;
        }
          
      } // end if (active[index] == minTime)        
      index++;

    } // end while(found && index < transNumber)         
    ++it;
  } // end while((cs = it.current()) != NULL && stateIndex < maxStNumber) 
  
  if(cs ==NULL)
  {
    finished = true;
  }

  delete active;
  delete nactive;
  delete delta;
}





