/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cstate.h"


//--------------------------------------------------------------------------
/** Creates an empty state.	*/

CState::CState()
{
	sTag = 0;
	pCount = 0;
	tCount = 0;
	mark = 0;
	tdelta = 0;
}


//--------------------------------------------------------------------------
/** Creates a state with \a tag for a net with \a pc places and
  \a tc transitions.	*/

CState::CState(int tag, int pc, int tc)
{
	sTag = tag;
	if(pc < 0 )
  {
	  mark = 0;
	  pCount = 0;
	}
	else
  {
	  pCount = pc;
	  mark = new int[pCount];
	}
  
	if(tc < 0 ) 
  {
	  tCount = 0;
	  tdelta = 0;
	}
	else
  {
	  tCount = tc;
	  tdelta = new int[tCount];
	}
		
}
		
		
//--------------------------------------------------------------------------
/** Destroys the state.	*/

CState::~CState()
{
	if(mark) delete [] mark;
	if(tdelta) delete [] tdelta;
	succ.clear();
}

//--------------------------------------------------------------------------
/** Creates a new state that is copy of \a source but it contains
  na empty list of successors.	*/

CState::CState(CState &source)
{
	sTag = source.sTag;
	pCount = source.pCount;
	tCount = source.tCount;
	int i;
	
	if(pCount > 0)	
	{
	  mark = new int[pCount];
	  for(i = 0; i < pCount; i++)
	  {
	    mark[i] = source.mark[i];
	  }
	}
	else 
  {
	  mark = 0;
	}
	
	if(tCount > 0) 
  {
	  tdelta = new int[tCount];
	  for(i = 0; i < tCount; i++) 
    {
	    tdelta[i] = source.tdelta[i];
	  }
	}
	else 
  {
		tdelta = 0;
	}
}


//--------------------------------------------------------------------------
/** Returns \a true if both states are equal; otherwise returns false.	*/

bool CState::equal(CState *source)
{
  if(pCount != source->pCount) return false;
  if(tCount != source->tCount) return false;

  //check marking
  for(int i = 0; i < pCount; i++)
  {
    if(mark[i] != source->mark[i]) return false;
  }
  
  //check delay
  for(int i = 0; i < tCount; i++)
  {
    if(tdelta[i] != source->tdelta[i]) return false;
  }
  
  return true;
}



//--------------------------------------------------------------------------
/** Returns the marking of the place with number \a index,
  or -1 if the index is out of range.
	The valid range is 0..(placeCount() - 1) inclusive. */ 

int CState::marking(int index)
{
	if(index < pCount)
	{
		return mark[index];
	}
	else
	{
		return -1;
	}
}


//--------------------------------------------------------------------------
/** Returns the dynamic delay of the transition with number \a index,
  or -1 if the index is out of range.
	The valid range is 0..(transCount() - 1) inclusive. */ 

int CState::delta(int index)
{
	if(index < tCount)
	{
		return tdelta[index];
	}
	else
	{
		return -1;
	}
}


//--------------------------------------------------------------------------
/** Sets the marking of the place with number \a index to \a tokens,
  or does nothing if the index is out of range or the new marking is negative.
	The valid range is 0..(placeCount() - 1) inclusive. */ 

void CState::setMarking(int index, int tokens)
{
	if((index < pCount) && (tokens >= 0))
	{
		mark[index] = tokens;
	}
}


//--------------------------------------------------------------------------
/** Sets the dynamic delay of the transition with number \a index to \a ndelta,
  or does nothing if the index is out of range. 
	The valid range is 0..(placeCount() - 1) inclusive. 
  Any negative delay value denotes the the corresponding transition is disable. */ 

void CState::setDelta(int index, int ndelta)
{
	int d = (ndelta < 0) ? -1 : ndelta;
	if(index < tCount)
	{
		tdelta[index] = d;
	}
}


//--------------------------------------------------------------------------
/** Returns description of the state: state tag, marking and delay vector. */

QString CState::info(bool withTime)
{
  //QString desc("state: ");
  QString desc("");
  desc += QString::number(sTag);
  desc += ":\nM = (";

  for(int i = 0; i < pCount; i++)
  {
    if(i != 0) desc += ", ";
    if(mark[i]>=0)
    {
      desc += QString::number(mark[i]);
    }
    else
    {
      desc += "oo"; // oo
    }
  }
  
  if(withTime)
  {
    desc += ")\nD = (";  
    for(int i = 0; i < tCount; i++)
    {
      if(i !=0) desc += ", ";
      if(tdelta[i] < 0)
      {
        desc += "-";
      }
      else
      {
        desc += QString::number(tdelta[i]);
      }
    }
  }
    
  desc += ")\n";

  return desc;
}


//--------------------------------------------------------------------------
/** Returns description of the state successors.	If \a withTime
  is equal to \a true, the description contains information about time delays. */

QString CState::successors(CNet* cnet, bool withTime)
{
  QString info;//("succesors: ");
//  info += QString::number(succ.size());
//  info += "\n";
  
  QMap<int,int>::const_iterator ci;
  ci = succ.begin();
  
  while(ci != succ.end())
  {
    info += "--(" + cnet->transition(ci.key())->name(); 
    if(withTime)
    {
      info += ", " + QString::number(tdelta[ci.key()]);
    }
    info += ")-->" + QString::number(ci.data()) +"\n"; 
    ci++;
  }
 
  return info;
}


 



