/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cpropertieseditor.h"
#include <qcanvas.h>
#include "../model/cplace.h"
#include "../model/ctransition.h"
#include "../model/carc.h"
#include "../view/cellipsewithlabels.h"
#include "../view/crectanglewithlabels.h"
#include "../view/clinewithlabels.h"


//--------------------------------------------------------------------------
/** Constructs a properties editor with parent widget \a parent and with
   editor caption \a caption. */

CPropertiesEditor::CPropertiesEditor(QWidget *parent, QString caption) :
   QDockWindow(parent, caption.latin1())
{
   setResizeEnabled(true);
   enabled = false;
   setCaption(caption);
   setCloseMode(QDockWindow::Always);

   // Create a widget stack layout
   widgetStack = new QWidgetStack(this);
   widgetStack->adjustSize();
   setWidget(widgetStack);
}


//--------------------------------------------------------------------------
/** Adds a new table to the property editor. Returns the page index or
  -1 on failure. */

int CPropertiesEditor::addTable(QString header0, QString header1, int rows)
{
   // Create a table.
   curTable = new QTable(rows,2,this);
   curTable->horizontalHeader()->setLabel(0, header0);
   curTable->horizontalHeader()->setLabel(1, header1);
   curTable->adjustSize();
   curTable->setColumnStretchable(1,true);
   curTable->setLeftMargin(0);   // to hide rows labels
   curTable->setSelectionMode(QTable::Single);
   curTable->setColumnWidth(0, 80);
   curTable->setColumnReadOnly(0, true);

   // Add the page to the widgetStack.
   tables.append(curTable);
   int index = widgetStack->addWidget(curTable);
   return index;
}


//--------------------------------------------------------------------------
/** Sets properties editor values after a place is selected. */

void CPropertiesEditor::placeSelected(CEllipseWithLabels* ellipse)
{
  if (!ellipse) return;

   widgetStack->raiseWidget(1);
   curTable = tables.at(1);
  
  CPlace* place = static_cast<CPlace*>(ellipse->modelPointer());
  if(!place) return; 
   
  curTable->setText(0,1,place->name());
  curTable->setText(1,1,QString::number(place->capacity()));
  curTable->setText(2,1,QString::number(place->marking())); 
  curTable->setText(3,1,QString::number(place->tag()));
}


//--------------------------------------------------------------------------
/** Sets properties editor values after a transition is selected. */

void CPropertiesEditor::transSelected(CRectangleWithLabels* rectangle)
{
  if(!rectangle) return;
  
  widgetStack->raiseWidget(2);
  curTable = tables.at(2);
  
  CTransition* trans = static_cast<CTransition*>(rectangle->modelPointer());
  if(!trans) return;
   
  curTable->setText(0,1,trans->name());
  curTable->setText(1,1,QString::number(trans->priority()));
  curTable->setText(2,1,QString::number(trans->time())); 
  curTable->setText(3,1,QString::number(trans->tag()));
}


//--------------------------------------------------------------------------
/** Sets properties editor values after the canvas is selected. */

void CPropertiesEditor::canvasSelected(CNetView* view)
{
   if (!view) return;

  widgetStack->raiseWidget(0);
  curTable = tables.at(0);

  curTable->setText(0,1,view->name());
  curTable->setText(1,1,QString::number((view->canvas())->width()));
  curTable->setText(2,1,QString::number((view->canvas())->height()));
}


//--------------------------------------------------------------------------
/** Sets properties editor values after an arc is selected. */

void CPropertiesEditor::arcSelected(CLineWithLabels* line)
{
  if (!line) return;

  widgetStack->raiseWidget(3);
  curTable = tables.at(3);
  
  CArc* arc = static_cast<CArc*>(line->modelPointer());
  if(!arc) return;

   curTable->setText(0,1,QString::number(arc->weight()));
}


//--------------------------------------------------------------------------
/** Updates properties of selected element. */

void CPropertiesEditor::propertyChanged(int /* x */, int /* y */)
{
   int id = -1;
   int a, b, c;
   QTable *table;
   
   id = widgetStack->id(widgetStack->visibleWidget());

   switch (id)
   {
      case 0:  // canvas properties
        table = tables.at(0);
        a = atoi(table->text(1,1));
        b = atoi(table->text(2,1));
        emit setCanvasProperties(table->text(0,1),a,b);
        break;
      
      case 1:  // place properties
        table = tables.at(1);
        a = atoi(table->text(1,1));
        b = atoi(table->text(2,1));
        c = atoi(table->text(3,1));  
        emit setPlaceProperties(table->text(0,1),a,b,c);
        break;

      case 2:  // transition properties
        table = tables.at(2);
        a = atoi(table->text(1,1));
        b = atoi(table->text(2,1));
        c = atoi(table->text(3,1));  
        emit setTransProperties(table->text(0,1),a,b,c);
        break;
      
      case 3:  // arc properties
         table = tables.at(3);
         a = atoi(table->text(0,1));
         emit setArcProperties(a);
         break;
      
      default:
         break;
   }
}
