/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cnet.h"

//--------------------------------------------------------------------------
/**
  Creates a CPlace, adds it to the net.
  The place marking is set to the initial marking.
  \param tag the place tag,
  \param name the place name,
  \param m the place initial marking,
  \param c the place capacity.
*/

void CNet::addPlace(int tag, QString name, int m, int c)
{
  CPlace* p;
  p = new CPlace(tag, name, m, c);
  places.append(p);
}


//--------------------------------------------------------------------------
/** Adds the place \a p to the net. */

void CNet::addPlace(CPlace* p)
{
  places.append(p);
}


//--------------------------------------------------------------------------
/**
  Creates a CTransition, adds it to the net..
  \param tag the transition tag,
  \param name the transition name,
  \param pr the transition priority.
*/

void CNet::addTransition(int tag, QString name, int pr)
{
  CTransition* t;
  t = new CTransition(tag, name, pr);
  transitions.append(t);
}


//--------------------------------------------------------------------------
/** Adds the transition \a t to the net. */

void CNet::addTransition(CTransition* t)
{
  transitions.append(t);
}


//--------------------------------------------------------------------------
/**
  Creates a CArcs, adds it to the net and returns \a true
  or returns \a false if such an arc cannot be created i.e., 
  there exists an arc with the same direction
  that connects these nodes.
  \param tag the arc tag,
  \param p the place node,
  \param t the transition node,
  \param d the arc direction,
  \param w the arc weight.
*/

bool CNet::addArc(int tag, CPlace* p, CTransition* t, CArcDir d, int w)
{
  CArc* a;

  // check whether there exists an arc between these nodes
  for(a = arcs.first(); a; a = arcs.next())
  {
    if((a->place() == p) && (a->transition() == t) && (a->direction() == d))
    {
      return false;
    }
  }

  if(d == adPT)
  {
    a = new CArc(tag, p, t, w);
  }
  else
  {
    a = new CArc(tag, t, p, w);
  }
  arcs.append(a);
  return true;
}


//--------------------------------------------------------------------------
/** Adds the arc \a a to the net and returns \a true or returns \a false
  if such an arc cannot be created i.e., 
  there exists an arc with the same direction
  that connects these nodes. */

bool CNet::addArc(CArc* a)
{
  CArc* i;

  // check whether there exists an arc between these nodes
  for(i = arcs.first(); i; i = arcs.next())
  {
    if((i->place() == a->place())
    && (i->transition() == a->transition())
    && (i->direction() == a->direction()))
    {
      return false;
    }
  }

  arcs.append(a);
  return true;
}


//--------------------------------------------------------------------------
/** Returns a pointer to the arc that goes from place \a p to transition \a t
  or  returns 0 if there is no such an arc. */

CArc* CNet::arc(CPlace* p, CTransition* t)
{
  CArc* i;
  for(i = arcs.first(); i; i = arcs.next())
  {
    if((i->place() == p) && (i->transition() == t)
    && (i->direction() == adPT))
      return i;
  }

  return 0;
}


//--------------------------------------------------------------------------
/** Returns a pointer to the arc that goes from transition \a t to place \a p
  or  returns 0 if there is no such an arc. */

CArc* CNet::arc(CTransition* t, CPlace* p)
{
  CArc* i;
  for(i = arcs.first(); i; i = arcs.next())
  {
    if((i->place() == p) && (i->transition() == t)
    && (i->direction() == adTP))
      return i;
  }

  return 0;
}


//--------------------------------------------------------------------------
/** Removes the place \a p, all arcs connected with it and returns \a true
  or returns \a false if there is no such a place. */

bool CNet::removePlace(CPlace* p)
{
  CPlace* i;
  CArc* a;

  if(p != 0)
  {
    // remove arcs
    a = arcs.first();
    while(a)
    {
      if(a->place() == p)
      {
        removeArc(a);
        //  start again so the collection has changed
        a = arcs.first();
      }
      else
        a = arcs.next();
    }

    // find and remove the place
    for(i = places.first(); i; i = places.next())
    {
      if(i == p)
      {
        delete p;
        places.remove();
        return true;
      }
    }
  }

  return false;
}


//--------------------------------------------------------------------------
/** Removes the transition \a t, all arcs connected with it and returns \a true
  or returns \a false if there is no such a transition. */

bool CNet::removeTransition(CTransition* t)
{
  CTransition* i;
  CArc* a;

  if(t != 0)
  {
    // remove arcs
    a = arcs.first();
    while(a)
    {
      if(a->transition() == t)
      {
        removeArc(a);
        // start again so the collection has changed
        a = arcs.first();
      }
      else
        a = arcs.next();
    }

    // find and remove the transition
    for(i = transitions.first(); i; i = transitions.next())
    {
      if(i == t) 
      {
        delete t;
        transitions.remove();
        return true;
      }
    }
  }

  return false;
}


//--------------------------------------------------------------------------
/** Removes the arc \a a and returns \a true or returns \a false
  if there is no such an arc. */

bool CNet::removeArc(CArc* a)
{
  CArc* i;

  if(a != 0)
  {
    // find and remove the arc
    for(i = arcs.first(); i; i = arcs.next())
    {
      if(i == a)
      {
        delete a;
        arcs.remove();
        return true;
      }
    }
  }

  return false;
}


//--------------------------------------------------------------------------
/** Removes all net nodes and arcs. */

void CNet::removeAllElements()
{
  CPlace* p;
  CTransition* t;
  CArc* a;

  for(a = arcs.first(); a; a = arcs.next())
  {
    delete a;
  }
  for(p = places.first(); p; p = places.next())
  {
    delete p;
  }
  for(t = transitions.first(); t; t = transitions.next())
  {
    delete t;
  }

  arcs.clear();
  places.clear();
  transitions.clear();
}


//--------------------------------------------------------------------------
/** Return the net incidence matrix, or 0 if the net is empty. */

int** CNet::matrix()
{
  CArc* a;

  int** matr;
  int i, j, columnCount, rowCount, in, out;
  
  sortElements();

  rowCount = places.count();
  columnCount = transitions.count();

  if((rowCount == 0) || (columnCount == 0))
    return 0;

  // create matrix
  matr = new int*[rowCount];
  for(i = 0; i < rowCount; i++)
    matr[i] = new int[columnCount];

  // fill matrix
  for(i = 0; i < rowCount; i++)
  {
    for(j = 0; j < columnCount; j++)
    {
      a = arc(transition(j), place(i));
      if(a == 0) // if there is no arc
			{
        in = 0;
			}
      else
			{
        in = a->weight();
			}
			
      a = arc(place(i), transition(j));
      if(a == 0) // if there is no arc
			{
        out = 0;
			}
      else
			{
        out = a->weight();
			}
      matr[i][j] = in - out;
    }
  }

  return matr;
}


//--------------------------------------------------------------------------
/** Return the net input matrix, or 0 if the net is empty.
  Remark: incidence matrix = input matrix - output matrix. */

int** CNet::inMatrix()
{
  CArc* a;
  int** matr;
  int i, j, columnCount, rowCount, in;
  
  sortElements();

  rowCount = places.count();
  columnCount = transitions.count();

  if((rowCount == 0) || (columnCount == 0))
    return 0;

  // create matrix
  matr = new int*[rowCount];
  for(i = 0; i < rowCount; i++)
      matr[i] = new int[columnCount];

  // fill matrix
  for(i = 0; i < rowCount; i++)
  {
    for(j = 0; j < columnCount; j++)
    {
      a = arc(transition(j), place(i));
      if(a == 0) // if there is no arc
			{
        in = 0;
			}
      else
			{
        in = a->weight();
			}
      if(in >= 0)
			{
        matr[i][j] = in;
			}
      else
			{
        matr[i][j] = 0;
			}
    }
  }

  return matr;
}


//--------------------------------------------------------------------------
/** Return the net output matrix, or 0 if the net is empty.
  Remark: incidence matrix = input matrix - output matrix. */

int** CNet::outMatrix()
{
  CArc* a;
  int** matr;
  int i, j, columnCount, rowCount, out;
  
  sortElements();

  rowCount = places.count();
  columnCount = transitions.count();

  if((rowCount == 0) || (columnCount == 0))
    return 0;

  // create matrix
  matr = new int*[rowCount];
  for(i = 0; i < rowCount; i++)
    matr[i] = new int[columnCount];

  // fill matrix
  for(i = 0; i < rowCount; i++)
  {
    for(j = 0; j < columnCount; j++)
    {
      a = arc(place(i), transition(j));
      if(a == 0) // if there is no arc
			{
				out = 0;
			}
      else 
			{
				out = a->weight();
			}
      if(out >= 0) 
			{
				matr[i][j] = out;
			}
      else
			{
        matr[i][j] = 0;
			}
    }
  }

  return matr;
}


//--------------------------------------------------------------------------
/** Sorts places and transitions using tags. */

void CNet::sortElements()
{
  QPtrList<CPlace> tempPlaces(places);
  CPlace *i, *j;
  int tag;
  int index;
  
  places.clear();
  for(i = tempPlaces.first(); i; i = tempPlaces.next())
  {
    tag = i->tag();
    index = 0;      
    for(j = places.first(); j; j = places.next())
    {
      if(j->tag() < tag) index++;
    }
    places.insert(index, i);
  }
  
  QPtrList<CTransition> tempTrans(transitions);
  CTransition *m, *n;
  
  transitions.clear();
  for(n = tempTrans.first(); n; n = tempTrans.next())
  {
    tag = n->tag();
    index = 0;      
    for(m = transitions.first(); m; m = transitions.next())
    {
      if(m->tag() < tag) index++;
    }
    transitions.insert(index, n);
  }
}


//--------------------------------------------------------------------------
/** Turn on the model simulation mode (initialises some variables). */

void CNet::startSimulation()
{
  int i, j;
  
  sortElements();
  clockValue = 0;
  
  placeCount = places.count();
  transCount = transitions.count();
 
  // initial marking
  curMarking = new int[placeCount];
  enableMarking = new int[placeCount];
  for(i = 0; i < placeCount; i++)
  {
    curMarking[i] = places.at(i)->marking();
    enableMarking[i] = curMarking[i];
  }
  
  // priorities
  priority = new int[transCount];
  for(i = 0; i < transCount; i++)
  {
    priority[i] = transitions.at(i)->priority();
  }
 
  enableTransition  = new int[transCount];
  dynamicDelay      = new int[transCount];
  
  for(i = 0; i < transCount; i++)
  {
    dynamicDelay[i] = 0;
  }
  
  firingMatrix = new int*[placeCount];
  inputMatrix  = new int*[placeCount];
  outputMatrix = new int*[placeCount];
  for(i = 0; i < placeCount; i++)
  {
    firingMatrix[i] = new int[transCount];
    inputMatrix[i]  = new int[transCount];
    outputMatrix[i] = new int[transCount];
  }
  
  // input and output matrices
  int ** tMatrix;
  
  tMatrix = inMatrix();
  for(i = 0; i < placeCount; i++)
  {
    for(j = 0; j < transCount; j++)
    {
      inputMatrix[i][j] = tMatrix[i][j];
    }
  }
  
  tMatrix = outMatrix();
  // initial matrix
  for(i = 0; i < placeCount; i++)
  {
    for(j = 0; j < transCount; j++)
    {
      outputMatrix[i][j] = tMatrix[i][j];
      firingMatrix[i][j] = tMatrix[i][j];
    }
  }
  
  // static delays
  staticDelay = new int[transCount];
  for(i = 0; i < transCount; i++)
  {
    staticDelay[i] = transitions.at(i)->time();
  }
    
  // suppose, all transitions are enabled
  for(i = 0; i < transCount; i++)
  {
    enableTransition[i] = 1;
  }    
     
  // check whether they are token enabled  
  bool enable;
  for(i = 0; i < transCount; i++)
  {
    enable = true;
    for(j = 0; j < placeCount; j++)
    {
      if(firingMatrix[j][i] > curMarking[j])
      {
        enable = false;
        break;
      }
    }
    
    if(!enable)
    {
      for(j = 0; j < placeCount; j++)
      {
        firingMatrix[j][i] = 0;
      }
      enableTransition[i] = 0;
      dynamicDelay[i] = -1;
    }
  }
  
  if(withPriorities)
  {
    int pCount;
    int maxPriority;
    // find shared input places
    for(i = 0; i < placeCount; i++)
    {
      pCount = 0;
      for(j = 0; j < transCount; j++)
      {
        if(firingMatrix[i][j]) pCount++;
      }
      
      if(pCount > 1)
      {
        // find maximum priority
        maxPriority = 0;
        for(j = 0; j < transCount; j++)
        {
          if((firingMatrix[i][j]) && (priority[j] > maxPriority))
          {
            maxPriority = priority[j];
          }
        }
        
        // unckeck disable transitions
        for(j = 0; j < transCount; j++)
        {
          if((firingMatrix[i][j]) && (priority[j] < maxPriority))
          {
            for(int k = 0; k < placeCount; k++)
            {
              firingMatrix[k][j] = 0;
            }
            enableTransition[j] = 0;
            dynamicDelay[j] = -1;
          }
        }
      }
    } // end for
  } // end withPriorities
  
  if(withTime)
  {
    // set dynamic delays
    for(i = 0; i < transCount; i++)
    {
      if(enableTransition[i]) dynamicDelay[i] = staticDelay[i];
    }
    
    // find minimal static delay
    i = 0;
    timeStep = dynamicDelay[0];
    while((dynamicDelay[i] == -1) && (i < transCount - 1))
    {
      i++;
      timeStep = dynamicDelay[i];
    }
    
    while(i < transCount - 1)
    {
      i++;
      if((dynamicDelay[i] != -1) && (dynamicDelay[i] < timeStep)) 
      {
        timeStep = dynamicDelay[i];
      }
    }
    
    if(timeStep != -1)
    {
      for(i = 0; i < transCount; i++)
      {
        if((dynamicDelay[i] != -1) && (dynamicDelay[i] > timeStep)) 
        {
          for(int k = 0; k < placeCount; k++)
          {
            firingMatrix[k][i] = 0;
          }
          enableTransition[i] = 0; // keep the dynamic delay
        }  
      }
      for(i = 0; i < transCount; i++)
      {
        if(dynamicDelay[i] != -1) dynamicDelay[i] -= timeStep;
      }
            
      clockValue += timeStep;
    }    
    
    // update transition dynamic delays
    CTransition *t;
    for(i = 0; i < transCount; i++)
    {
      t = transitions.at(i);
      t->setWaitingTime(dynamicDelay[i]);
    }    
  } // end withTime  
}


//--------------------------------------------------------------------------
/** Turn off the model simulation mode. */

void CNet::stopSimulation()
{
  delete [] curMarking;
  delete [] enableTransition;
  delete [] dynamicDelay;
  delete [] priority;
  delete [] staticDelay;
  delete [] enableMarking;
  
  for(int i = 0; i < placeCount; i++)
  {
    delete [] inputMatrix[i];
    delete [] outputMatrix[i];
    delete [] firingMatrix[i];
  }
  
  delete [] inputMatrix;
  delete [] outputMatrix;
  delete [] firingMatrix; 
}


//--------------------------------------------------------------------------
/** Returns \a true if the transition is enables or \a false otherwise. */

bool CNet::enable(CTransition* t)
{
  int index = transitions.find(t);
  
  if((enableTransition[index] == 1) || (enableTransition[index] == 3)) return true;
  else return false;
}


//--------------------------------------------------------------------------
/** Checks off the transition to fired. */

void CNet::checkOff(CTransition* t, bool yes)
{
  int i = transitions.find(t);
  int j, k;
  
  if(yes)
  {
    if(enableTransition[i] == 1) enableTransition[i] = 3;
        
    for(j = 0; j < placeCount; j++)
    {
      firingMatrix[j][i] = 0;
      enableMarking[j] = enableMarking[j] - outputMatrix[j][i];
    }     
  }
  else
  {
    if(enableTransition[i] == 3) enableTransition[i] = 1;
    
    for(j = 0; j < placeCount; j++)
    {
      enableMarking[j] = enableMarking[j] + outputMatrix[j][i];
    }
    
    for(k = 0; k < transCount; k++)
    {
      if(enableTransition[k] == 2) enableTransition[k] = 1;
      for(j = 0; j < placeCount; j++)
      {
        firingMatrix[j][k] = outputMatrix[j][k];
      }
    }    
  }
  
  // check whether other transitions are still token enabled  
  bool enable;
  for(k = 0; k < transCount; k++)
  {
    if(enableTransition[k] == 1)
    {
      enable = true;
      for(j = 0; j < placeCount; j++)
      {
        if(firingMatrix[j][k] > enableMarking[j])
        {
          enable = false;
          break;
        }
      }
      
      if(!enable)
      {
        for(j = 0; j < placeCount; j++)
        {
          firingMatrix[j][k] = 0;
        }
        enableTransition[k] = 2;
      }
    }
  }  
}


//--------------------------------------------------------------------------
/**  Fires the checked off transitions and finds new active ones. */

void CNet::fire()
{
  int i, j;  
  
  for(i = 0; i < transCount; i++)
  {
    if(enableTransition[i] == 3)
    {
      for(j = 0; j < placeCount; j++)
      {
        curMarking[j] = 
          curMarking[j] - outputMatrix[j][i] + inputMatrix[j][i]; 
      }
      dynamicDelay[i] = -1;
    }
  } 
   
  for(i = 0; i < placeCount; i++) 
  {
    places.at(i)->setMarking(curMarking[i]); 
    enableMarking[i] = curMarking[i];
  }
  
  // initial firing matrix
  for(i = 0; i < placeCount; i++)
  {
    for(j = 0; j < transCount; j++)
    {
      firingMatrix[i][j] = outputMatrix[i][j];
    }
  }
  
  // suppose, all transitions are enabled
  for(i = 0; i < transCount; i++)
  {
    enableTransition[i] = 1;
  }    
     
  // check whether they are token enabled  
  bool enable;
  for(i = 0; i < transCount; i++)
  {
    enable = true;
    for(j = 0; j < placeCount; j++)
    {
      if(firingMatrix[j][i] > curMarking[j])
      {
        enable = false;
        break;
      }
    }
    
    if(!enable)
    {
      for(j = 0; j < placeCount; j++)
      {
        firingMatrix[j][i] = 0;
      }
      enableTransition[i] = 0;
      dynamicDelay[i] = -1;
    }
  }
  
  if(withPriorities)
  {
    int pCount;
    int maxPriority;
    // find shared input places
    for(i = 0; i < placeCount; i++)
    {
      pCount = 0;
      for(j = 0; j < transCount; j++)
      {
        if(firingMatrix[i][j]) pCount++;
      }
      
      if(pCount > 1)
      {
        // find maximum priority
        maxPriority = 0;
        for(j = 0; j < transCount; j++)
        {
          if((firingMatrix[i][j]) && (priority[j] > maxPriority))
          {
            maxPriority = priority[j];
          }
        }
        
        // unckeck disable transitions
        for(j = 0; j < transCount; j++)
        {
          if((firingMatrix[i][j]) && (priority[j] < maxPriority))
          {
            for(int k = 0; k < placeCount; k++)
            {
              firingMatrix[k][j] = 0;
            }
            enableTransition[j] = 0;
            dynamicDelay[j] = -1;
          }
        }
      }
    } // end for
  } // end withPriorities
  
  if(withTime)
  {
    // set dynamic delays
    for(i = 0; i < transCount; i++)
    {
      if((enableTransition[i]) && (dynamicDelay[i] == -1))
      {
        dynamicDelay[i] = staticDelay[i];
      }
    }
    
    // find minimal static delay
    i = 0;
    timeStep = dynamicDelay[0];
    while((dynamicDelay[i] == -1) && (i < transCount - 1))
    {
      i++;
      timeStep = dynamicDelay[i];
    }
    
    while(i < transCount - 1)
    {
      i++;
      if((dynamicDelay[i] != -1) && (dynamicDelay[i] < timeStep)) 
      {
        timeStep = dynamicDelay[i];
      }
    }
    
    if(timeStep != -1)
    {
      for(i = 0; i < transCount; i++)
      {
        if((dynamicDelay[i] != -1) && (dynamicDelay[i] > timeStep)) 
        {
          for(int k = 0; k < placeCount; k++)
          {
            firingMatrix[k][i] = 0;
          }
          enableTransition[i] = 0; // keep the dynamic delay
        }  
      }
      for(i = 0; i < transCount; i++)
      {
        if(dynamicDelay[i] != -1) dynamicDelay[i] -= timeStep;
      }
      
      clockValue += timeStep;
    }    
    
    // update transition dynamic delays
    CTransition *t;
    for(i = 0; i < transCount; i++)
    {
      t = transitions.at(i);
      t->setWaitingTime(dynamicDelay[i]);
    } 
  } 
}


