/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CNET_H
#define CNET_H

#include <qptrlist.h>
#include "carc.h"


/**
CNet class objects represent low level nets.
Such an object contains list of places, list of transition,
list of arc and it manages these elements.
*/

class CNet
{
protected:
  QPtrList<CPlace>      places;
  QPtrList<CTransition> transitions;
  QPtrList<CArc>        arcs;

  // simulation
  bool withPriorities;
  bool withTime;
  int **firingMatrix;
  int **inputMatrix;
  int **outputMatrix;
  int *curMarking;
  int *dynamicDelay;
  int *staticDelay;
  int *enableTransition; //0 - disable, 1 - enable, 2 - unchecked, 3 - fireable
  int *enableMarking;
  int *priority;
  int placeCount;
  int transCount;
  int timeStep;
  int clockValue;

public:
  /** Creates an empty net.  */
  CNet()
  { withPriorities = false;
    withTime = false;
  };

  /** Destroys the net.  */
  ~CNet(){ removeAllElements(); }

  // adding elements
  void addPlace(int tag, QString name, int m = 0, int c = 0);
  void addPlace(CPlace* p);
  void addTransition(int tag, QString name, int pr = 0);
  void addTransition(CTransition* t);
  bool addArc(int tag, CPlace* p, CTransition* t, CArcDir d, int w = 1);
  bool addArc(CArc* a);

  // removing elements
  bool removePlace(CPlace* p);
  bool removeTransition(CTransition* t);
  bool removeArc(CArc* a);
  void removeAllElements();

  /** Returns a pointer to the place at position \a index in the list,
  or 0 if the index is out of range. The valid range is 0..(places.count() - 1) inclusive. */
  CPlace* place(int index){ return places.at(index); }

  /** Returns a pointer to the tranition at position \a index in the list,
  or 0 if the index is out of range. The valid range is 0..(tranitions.count() - 1) inclusive. */
  CTransition* transition(int index){ return transitions.at(index); }

  /** Returns a pointer to the arc at position \a index in the list,
  or 0 if the index is out of range. The valid range is 0..(arcs.count() - 1) inclusive. */
  CArc* arc(int index){ return arcs.at(index); }

  CArc* arc(CPlace* p, CTransition* t);
  CArc* arc(CTransition* t, CPlace* p);

  /** Returns the number of the net places.  */
  int placesCount() const { return places.count(); };

  /** Returns the number of the net transitions.  */
  int transitionsCount() const { return transitions.count(); };

  /** Returns the number of the net arcs.  */
  int arcsCount() const { return arcs.count(); };

  /** Returns the number of the net elements.  */
  int count() const { return (places.count() + transitions.count() + arcs.count()); };

  // incident matrix
  int** matrix();
  int** inMatrix();
  int** outMatrix();

  /** Sets the simulation parameters. */
  void setSimulationParameters(bool priorities, bool time)
  { 
    withPriorities = priorities;
    withTime = time; 
  }
  
  void sortElements();
  void startSimulation();
  void stopSimulation();
  bool enable(CTransition* t);
  void checkOff(CTransition* t, bool yes);
  void fire();
  
  /** Returns the number of time-units the clock should be moved.  */
  int  clock() { return clockValue; }
};


#endif
