/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cxmlgenerator.h"
#include "../view/cellipsewithlabels.h"
#include "../view/crectanglewithlabels.h"
#include "../view/csegment.h"
#include "../view/clinewithlabels.h"
#include <qptrlist.h>

//--------------------------------------------------------------------------
/** Constructs an XML generator with \a device and \a codec. */

CXMLGenerator::CXMLGenerator(QIODevice *device, int indent, QTextCodec *codec)
{
  indentSize = indent;
  level = 0;

  output.setDevice(device);
  if(codec == 0)
  {
    codec = QTextCodec::codecForName("ISO8859-2");
  }
  output.setCodec(codec);
  output << "<?xml version=\"1.0\" encoding=\""
         << escapeXML(codec->mimeName()) << "\"?>\n";
}


//--------------------------------------------------------------------------
/** The function is used to escape special characters in text and attribute
  values. */

QString CXMLGenerator::escapeXML(const QString &text)
{
  QString xml = text;
  xml.replace("&", "&amp;");
  xml.replace("<", "&lt;");
  xml.replace(">", "&gt;");
  xml.replace("\'", "&apos;");
  xml.replace("\"", "&quot;");
  return xml;
}


//--------------------------------------------------------------------------
/** Writes a new open tag \a name with list of attributes \a attrList. */

void CXMLGenerator::writeOpenTag(const QString& name, const QStringList& attrList)
{
  QString xml = QString().fill(' ', indentSize * level);
  xml +=  "<" + name;

  int i = attrList.count();
  for(int j = 0; j < i; j++)
  {
    xml += " " + attrList[j];
  }
  xml += ">";

  level++;
  output << xml << endl;
}


//--------------------------------------------------------------------------
/** This is an overloaded member function, provided for convenience.
  Writes a new open tag \a name without any attributes. */

void CXMLGenerator::writeOpenTag(const QString& name)
{
  QString xml = QString().fill(' ', indentSize * level);
  xml +=  "<" + name + ">";
  level++;
  output << xml << endl;
}


//--------------------------------------------------------------------------
/** Closes the tag \a name. */

void CXMLGenerator::writeCloseTag(const QString& name)
{
  level--;
  QString xml = QString().fill(' ', indentSize * level);
  xml += "</" + name + ">";
  output << xml << endl;
}


//--------------------------------------------------------------------------
/** Writes an open and close tag \a name with \a text between them.
  The open tag is written together with a list of attributes \a attrList.
  If the \a newLine is \a true a new line is put after the open tag and
  the \a text. */

void CXMLGenerator::writeTag(const QString& name, const QStringList& attrList,
                             const QString& text, bool newLine)
{
  QString xml = QString().fill(' ', indentSize * level);
  xml +=  "<" + name;

  int i = attrList.count();
  for(int j = 0; j < i; j++)
  {
    xml += " " + attrList[j];
  }
  xml += ">";
  output << xml;

  if(newLine)
  {
    output << endl;
    level++;
    xml = QString().fill(' ', indentSize * level);
  }
  else
  {
    xml = "";
  }

  xml += escapeXML(text);
  output << xml;

  if(newLine)
  {
    output << endl;
    level--;
    xml = QString().fill(' ', indentSize * level);
  }
  else
  {
    xml = "";
  }

  xml += "</" + name + ">";
  output << xml << endl;
}


//--------------------------------------------------------------------------
/** This is an overloaded member function, provided for convenience.
  Writes an open and close tag \a name with \a text between them.
  The open tag is written without any attributes.
  If the \a newLine is \a true a new line is put after the open tag and
  the \a text. */

void CXMLGenerator::writeTag(const QString& name,
                             const QString& text, bool newLine)
{
  QString xml = QString().fill(' ', indentSize * level);
  xml +=  "<" + name + ">";
  output << xml;

  if(newLine)
  {
    output << endl;
    level++;
    xml = QString().fill(' ', indentSize * level);
  }
  else
  {
    xml = "";
  }

  xml += escapeXML(text);
  output << xml;

  if(newLine)
  {
    output << endl;
    level--;
    xml = QString().fill(' ', indentSize * level);
  }
  else
  {
    xml = "";
  }

  xml += "</" + name + ">";
  output << xml << endl;
}


//--------------------------------------------------------------------------
/** Writes a new open-close tag \a name with list of attributes \a attrList. */

void CXMLGenerator::writeOpenCloseTag(const QString& name,
                                      const QStringList& attrList)
{
  QString xml = QString().fill(' ', indentSize * level);
  xml +=  "<" + name;

  int i = attrList.count();
  for(int j = 0; j < i; j++)
  {
    xml += " " + attrList[j];
  }
  xml += "/>";

  output << xml << endl;
}


//--------------------------------------------------------------------------
/** Writes the net to the file. */

void CXMLGenerator::writeNet(CNetView* netView) //QCanvas* canvas, QString name)
{

  QStringList attrList;
  QString attr;
  QCanvas* canvas = netView->canvas();
  QCanvasItemList items = canvas->allItems();
  QCanvasItemList::iterator it;
  CPlace* placePtr;
  CTransition* transPtr;

  // atrybuty dla sieci
  attr = "name=\"";
  attr += netView->name();
  attr += "\"";
  attrList.append(attr);
  attr = "width=\"";
  attr += QString("%1").arg(canvas->width());
  attr += "\"";
  attrList.append(attr);
  attr = "height=\"";
  attr += QString("%1").arg(canvas->height());
  attr += "\"";
  attrList.append(attr);

  writeOpenTag("net", attrList);
  attrList.clear();

  // places
  writeOpenTag("places");

  it = items.begin();
  QPtrList<CEllipseWithLabels> ellipses;
  CEllipseWithLabels *ellipse, *e;
  int tag, index;
  
  // sort places
  for (; it != items.end(); it++)
  {
    if((*it)->rtti() == 10005)  // CEllipseWithLabels
    {
      ellipse = static_cast<CEllipseWithLabels*>(*it);
      tag = ellipse->tag();
      index = 0;      
      for(e = ellipses.first(); e; e = ellipses.next())
      {
        if(e->tag() < tag) index++;
      }
      ellipses.insert(index, ellipse);
    }
  }
      
  for(e = ellipses.first(); e; e = ellipses.next())
  {
    placePtr = static_cast<CPlace*>(e->modelPointer());
    attr = "name=\"";
    attr += placePtr->name();
    attr += "\"";
    attrList.append(attr);
    attr = "marking=\"";
    attr += QString("%1").arg(placePtr->initialMarking());
    attr += "\"";
    attrList.append(attr);
    attr = "capacity=\"";
    attr += QString("%1").arg(placePtr->capacity());
    attr += "\"";
    attrList.append(attr);
    attr = "x=\"";
    attr += QString("%1").arg(int(e->x()));
    attr += "\"";
    attrList.append(attr);
    attr = "y=\"";
    attr += QString("%1").arg(int(e->y()));
    attr += "\"";
    attrList.append(attr);
    attr = "namex=\"";
    attr += QString("%1").arg(int(e->label(0)->x()));
    attr += "\"";
    attrList.append(attr);
    attr = "namey=\"";
    attr += QString("%1").arg(int(e->label(0)->y()));
    attr += "\"";
    attrList.append(attr);

    writeOpenCloseTag("place", attrList);
    attrList.clear();
  }

  writeCloseTag("places");

  // transitions
  writeOpenTag("transitions");
  
  it = items.begin();
  QPtrList<CRectangleWithLabels> rectangles;
  CRectangleWithLabels *rectangle, *r;
  
  // sort transitions
  for (; it != items.end(); it++)
  {
    if((*it)->rtti() == 10004)  // CRectangleWithLabels
    {
      rectangle = static_cast<CRectangleWithLabels*>(*it);
      tag = rectangle->tag();
      index = 0;      
      for(r = rectangles.first(); r; r = rectangles.next())
      {
        if(r->tag() < tag) index++;
      }
      rectangles.insert(index, rectangle);
    }
  }
  
  for(r = rectangles.first(); r; r = rectangles.next())
  {
    transPtr = static_cast<CTransition*>(r->modelPointer());
    attr = "name=\"";
    attr += transPtr->name();
    attr += "\"";
    attrList.append(attr);
    attr = "priority=\"";
    attr += QString("%1").arg(transPtr->priority());
    attr += "\"";
    attrList.append(attr);
    attr = "time=\"";
    attr += QString("%1").arg(transPtr->time());
    attr += "\"";
    attrList.append(attr);
    attr = "x=\"";
    attr += QString("%1").arg(r->x());
    attr += "\"";
    attrList.append(attr);
    attr = "y=\"";
    attr += QString("%1").arg(r->y());
    attr += "\"";
    attrList.append(attr);
    attr = "namex=\"";
    attr += QString("%1").arg(int(r->label(0)->x()));
    attr += "\"";
    attrList.append(attr);
    attr = "namey=\"";
    attr += QString("%1").arg(int(r->label(0)->y()));
    attr += "\"";
    attrList.append(attr);
    attr = "prtimex=\"";
    attr += QString("%1").arg(int(r->label(1)->x()));
    attr += "\"";
    attrList.append(attr);
    attr = "prtimey=\"";
    attr += QString("%1").arg(int(r->label(1)->y()));
    attr += "\"";
    attrList.append(attr);

    writeOpenCloseTag("transition", attrList);
    attrList.clear();
  }

  writeCloseTag("transitions");

  // arcs
  writeOpenTag("arcs");

  it = items.begin();
  for (; it != items.end(); it++)
  {
    // segments z zawieszka 0 (kady luk ma jeden taki!)
    if(((*it)->rtti() == 10013) && (((CSegment *)(*it))->tag() == 0)) // CSegment
    {
      CSegment *s;
      if((s = (CSegment*)(*it)))
      {
        CLineWithLabels *arc;
        if((arc = (CLineWithLabels*)(s->owner())))
        {
          if(CArc* arcPtr = static_cast<CArc* >(arc->modelPointer()))
          {
            attr = "type=\"";
            if(arcPtr->direction() == adTP)
            {
              attr += "in";
            }
            else
            {
              attr += "out";
            }
            attr += "\"";
            attrList.append(attr);
            attr = "weight=\"";
            attr += QString("%1").arg(arcPtr->weight());
            attr += "\"";
            attrList.append(attr);
            attr = "place=\"";
            attr += (arcPtr->place())->name();
            attr += "\"";
            attrList.append(attr);
            attr = "transition=\"";
            attr += (arcPtr->transition())->name();
            attr += "\"";
            attrList.append(attr);
            attr = "weightx=\"";
            attr += QString("%1").arg(int(arc->label(0)->x()));
            attr += "\"";
            attrList.append(attr);
            attr = "weighty=\"";
            attr += QString("%1").arg(int(arc->label(0)->y()));
            attr += "\"";
            attrList.append(attr);

            writeOpenTag("arc", attrList);
            attrList.clear();

            // break points
            int n = arc->breaksCount();
            QPoint p;
            for(int i = 0; i < n; i++)
            {
              p = arc->breakpoint(i);
              attr = "x=\"";
              attr += QString("%1").arg(p.x());
              attr += "\"";
              attrList.append(attr);
              attr = "y=\"";
              attr += QString("%1").arg(p.y());
              attr += "\"";
              attrList.append(attr);
              writeOpenCloseTag("breakpoint", attrList);
              attrList.clear();
            }

            writeCloseTag("arc");
          }
        }
      }
    }
  }

  writeCloseTag("arcs");
  writeCloseTag("net");
}
