#ifndef _SVG_DOCUMENT
#define _SVG_DOCUMENT

#include <qvaluevector.h>

#include "XMLTag.h"
#include "IXMLTagStream.h"
#include "SVGElement.h"

#define SVG_DOCTYPE_DECL	"DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\""

namespace SVGOutput
{

/** Represents SVG Document. */

class SVGDocument : public IXMLTagStream
{
	float width, height;
	
	QRect viewBox;

	QValueVector<XMLTag *> tags;
	QValueVector<XMLTag *>::Iterator it;

	QValueVector<SVGElement *> elements;

	QString externalStyleSheet;
	QString externalScript, scriptType;
	QString entryPoint;
	
	QString units;

	QString viewBoxToString()
	{
	    static char buff[71];
	    sprintf(buff, "%i, %i, %i, %i", viewBox.x(), viewBox.y(), 
			viewBox.width(), viewBox.height());

	    return QString(buff);
	}
	
public:
	/** Creates the default document. */
	SVGDocument(): width(0), height(0), units("")  { }

	/** Creates document of size: @a width and @a height expressed in @a units */
	SVGDocument(float width, float height, QString units)
		: width(width), height(height), units(units) { }
	
	~SVGDocument() { }

	/** Adds new element to the document. */
	void addElement(SVGElement *element){ elements.push_back(element); }

	/** Sets link to external CSS StyleSheet. */
	void setExternalStyleSheet(QString url){ externalStyleSheet = url; }

	/** Gets link to external CSS StyleSheet. */
	QString getExternalStyleSheet() { return externalStyleSheet; }

	/** Sets link to external script file. */
	void setExternalScriptUrl(QString url){ externalScript = url;	}

	/** Gets link to external script file. */
	QString getExternalScriptUrl() { return externalScript; }

	/** Sets external script type. */
	void setExternalScriptType(QString type){ scriptType = type; }

	/** Gets external script type. */
	QString getExternalScriptType() { return scriptType; }

	/** Sets viewBox of the document root.
	 "It is often desirable to specify that a given set of 
	 graphics stretch to fit a particular container element. 
	 The viewBox attribute provides this capability." */
	void setViewBox(int x, int y, int width, int height)
	{
	    viewBox.setX(x); viewBox.setY(y);
	    viewBox.setWidth(width); viewBox.setHeight(height);
	}
	
  /** Sets viewBox of the document root. */
	void setViewBox(QRect rect){ viewBox = rect; }

	/** Specifies script entry function called in 'onload' event */
	void setScriptEntryPoint(QString function){ entryPoint = function; }
	
	/** Returns script entry point */
	QString getScriptEntryPoint() { return entryPoint; }
		
	void init() 
	{ 
		XMLTag *xml = new XMLTag("xml", XMLTag::ttInstruction);
		xml->addAttribute("version", "1.0");
		xml->addAttribute("encoding", "iso-8859-1");
		tags.push_back(xml);

		XMLTag *document = new XMLTag(SVG_DOCTYPE_DECL, XMLTag::ttDocumentType);
		tags.push_back(document);

		if(!externalStyleSheet.isEmpty())
		{
			XMLTag *styleSheet = new XMLTag("xml-stylesheet", XMLTag::ttInstruction);
			styleSheet->addAttribute("href", externalStyleSheet);
			styleSheet->addAttribute("type", "text/css");

			tags.push_back(styleSheet);
		}

		XMLTag *svg = new XMLTag("svg", XMLTag::ttOpening);

		if(!entryPoint.isEmpty())
		    svg->addAttribute("onload", entryPoint);
		
		svg->addAttribute("xmlns", "http://www.w3.org/2000/svg");
		svg->addAttribute("xmlns:xlink", "http://www.w3.org/1999/xlink");
		
		if(!viewBox.isEmpty())
		    svg->addAttribute("viewBox", viewBoxToString());
		
		if(width)
		    svg->addAttribute("width", QString("%1%2").arg(width).arg(units));
		    
		if(height)    
		    svg->addAttribute("height", QString("%1%2").arg(height).arg(units));
		    
		svg->addAttribute("x", "0");
		svg->addAttribute("y", "0");

		tags.push_back(svg);

		if(!externalScript.isEmpty())
		{
			XMLTag *script = new XMLTag("script", XMLTag::ttElement);
			    
			script->addAttribute("xlink:href", externalScript);
			
			if(!scriptType.isEmpty())
			    script->addAttribute("type", scriptType);

			tags.push_back(script);
		}

		QValueVector<SVGElement *>::iterator iter = elements.begin();
		while(iter != elements.end())
		{
			SVGElement *elem = *iter;
			elem->init();

			while(elem->hasNext())
				tags.push_back(elem->next());
			iter++;
		}

		tags.push_back(new XMLTag("svg", XMLTag::ttClosing));

		it = tags.begin();
	}

	void reset() { it = tags.begin(); }
	
	XMLTag *next() { return *it++; }
	bool hasNext() { return (it != tags.end()); }
};

} // end of namespace SVGOutput

#endif // _SVG_DOCUMENT
