#ifndef _SVG_ELEMENT
#define _SVG_ELEMENT

#include <qstring.h>

#include "XMLTag.h"

namespace SVGOutput
{

/**
 SVGElement is a base class for all elements (eg. shapes)
 of SVG Document.
 
 Implements IXMLTagStream. This allows XMLTextWriter to read
 tag data from SVGElement and it's descendants.
*/
class SVGElement : public IXMLTagStream
{
private:
	QString styleClass;
	QString id;
	
	int x, y;

protected:
	/** Specifies whether stream was initialized. */
	bool initialized;

public:
	/** Creates element and places it in default position (0, 0). */
	SVGElement(): x(0), y(0) { }

	/** Creates element placed in user specified posioton. */
	SVGElement(int x, int y): x(x), y(y) { }

	/** Virtual destructor to be overriden by descendant classes. */
	virtual ~SVGElement() { }
	
	/** Initializes object as xml stream. */
	virtual void init() { initialized = true; }

	/** Resets object as xml stream. */
	virtual void reset() { initialized = true; }

	/** Retrieves next tag in the stream */
	virtual XMLTag *next()
	{
		if(initialized)
		{
			// Default tag
			XMLTag *tag = new XMLTag("", XMLTag::ttElement);

			// Add id if it has one
			if(!id.isEmpty())
				tag->addAttribute("id", id);

			// Add style class name if present
			if(!styleClass.isEmpty())
				tag->addAttribute("class", styleClass);

			// Add svg shape position
			tag->addAttribute("x", QString("%1").arg(x));
			tag->addAttribute("y", QString("%1").arg(y));

			// Only one tag to read, stream will be  non-readable 
			// (until reset()) upon return from next()
			initialized = false;
			return tag;
		}
		else
			// Stream is not initialized
			return NULL;
	}

	/** True if there are more tags to read from the stream. */
	virtual bool hasNext() { return initialized; }

	/** Returns Id of this element. */
	QString getId() { return id; }

	/** Sets Id of this element. */
	void setId(QString id) { this->id = id; }

	/** Retrieves x coordinate of this element's position. */
	int getX() { return x; }

	/** Sets x posiotion of this element. */
	void setX(int x) { this->x = x; }

	/** Retrieves y coordinate of this element's position. */
	int getY() { return y; }

	/** Sets y posiotion of this element. */
	void setY(int y) { this->y = y; }

	/** Returns CSS style class for this element */
	QString getStyleClass() { return styleClass; }

	/** Sets CSS style class of this element */
	void setStyleClass(QString styleClass) { this->styleClass = styleClass; }
};

} // end of namespace SVGOutput

#endif // _SVG_ELEMENT
