#ifndef _SVG_POLYLINE
#define _SVG_POLYLINE

#include <qstring.h>
#include "SVGElement.h"

#include <qvaluevector.h>

namespace SVGOutput
{

/**
 Represents 'polyline' element of SVG Document.
 The 'polyline' element defines a set of connected
 straight line segments. 
 Typically, 'polyline' elements define open shapes.
*/

class SVGPolyline : public SVGElement
{
	/** Stores point data. */
	struct point 
	{ 
	    int x; int y; 	

	    point(): x(0), y(0) { }	    
	    point(int x, int y): x(x), y(y) { }
	};
	
	QValueVector<point> points;

	int weight;
	bool arc;
	
	QString place, transition;
	QString direction;

	QString toString(point p)
	{
	    return QString("%1,%2").arg(p.x).arg(p.y);
	}
	
	void validate() { }
	
public:
	/** Creates empty polyline. */
	SVGPolyline() 
	{ 
	    setStyleClass("polyline"); 
	    
	    weight = 0;
	    arc = false;
	}

	XMLTag *next()
	{
		if(initialized)
		{
			validate();

			XMLTag *tag = SVGElement::next();
			tag->setName("polyline");
			
			tag->removeAttribute("x");
			tag->removeAttribute("y");

			QString path;
			
			QValueVector<point>::iterator it = points.begin();
			while(it != points.end())
			{
			    path += toString(*it);
			    path += " ";
			    
			    it++;
			}
			
			tag->addAttribute("points", path);
			
			if(isArc())
			{
			    tag->addAttribute("type", "arc");
			    tag->addAttribute("weight", QString("%1").arg(weight));
			    tag->addAttribute("place", place);
			    tag->addAttribute("transition", transition);
			    tag->addAttribute("target", direction);
			}

			return tag;
		}
		else
			return NULL;
	}
	
	/** Adds new point. */
	void insertPoint(int x, int y)
	{
	    points.push_back(point(x, y));
	}
	
	/** Retrieves x coordinate of point at @a index. */
	int getPointX(int index) { return points[index].x; }

	/** Retrieves y coordinate of point at @a index. */
	int getPointY(int index) { return points[index].y; }
	
	/** Returns number of points in a polyline. */
	int length() { return points.size(); }

	/** Sets weight of arc represented by this polyline. */	
	void setWeight(int weight) { this->weight = weight; }
	
	/** Gets weight of arc represented by this document. */
	int getWeight() { return weight; }	

	/** Sets whether this polyline represents arc of a petri net. */
	void setArc(bool value) { arc = value; }
	
	/** Gets information whether this polyline represents arc of a pteri net. */
	bool isArc() { return arc; }
	
	/** Sets xml place id of place associated with this arc. */
	void setPlace(QString placeId) { place = placeId; }
	
	/** Gets xml place id. */
	QString getPlace() { return place; }
	
	/** Sets xml transition id of transition associated with this arc. */
	void setTransition(QString transitionId) { transition = transitionId; }
	
	/** Gets xml transiotion id. */
	QString getTransition() { return transition; }
	
	/** Sets arc direction: 'place' or 'transiotion'. */
	void setDirection(QString direction) { this->direction = direction; }
	
	/** Gets arc direction: 'place' or 'transiotion'. */	
	QString getDirection() { return direction; }
};

} // end of namespace SVGOutput

#endif // _SVG_RECTANGLE
