#ifndef _SVG_RECTANGLE
#define _SVG_RECTANGLE

#include <qstring.h>
#include "SVGElement.h"

namespace SVGOutput
{

/**
 Represents 'rect' element of SVG Document.
 The 'rect' element defines a rectangle which is axis-aligned 
 with the current user coordinate system. Rounded rectangles 
 can be achieved by setting appropriate values for attributes rx and ry. 
 */
class SVGRectangle : public SVGElement
{
	int width, height;
	int rx, ry;
	
	int priority, time;
	bool transition;
	
	void validate()
	{
		if(width < 0)
			width = 0;

		if(height < 0)
			height = 0;

		if(rx < 0)
			rx = 0;

		if(ry < 0)
			ry = 0;
	}
	
	void initialize()
	{
	    setStyleClass("rectangle");
	    rx = 0; ry = 0;
	    
	    priority = time = 0;
	    transition = false;
	}
public:
	/** Creates default rectangle. */
	SVGRectangle(): width(0), height(0) { initialize(); }

	/** Creates rectangle using uder specified data. */
	SVGRectangle(int x, int y, int width, int height)
		: SVGElement(x, y), width(width), height(height) { initialize(); }

	XMLTag *next()
	{
		if(initialized)
		{
			validate();

			XMLTag *tag = SVGElement::next();
			tag->setName("rect");

			tag->addAttribute("width", QString("%1").arg(width));
			tag->addAttribute("height", QString("%1").arg(height));
			tag->addAttribute("rx", QString("%1").arg(rx));
			tag->addAttribute("ry", QString("%1").arg(ry));

			if(isTransition())
			{
			    tag->addAttribute("type", "transition");
			    tag->addAttribute("priority", QString("%1").arg(priority));
			    tag->addAttribute("time", QString("%1").arg(time));
			}

			return tag;
		}
		else
			return NULL;
	}

	/** 
	 Gets the x-axis radius of the ellipse used to 
	 round off the corners of the rectangle.
	 */
	int getRoundingX() { return rx; }

	/** Sets the x-axis radius of the ellipse. */
	void setRoundingX(int rx) { this->rx = rx; }

	/** 
	 Gets the y-axis radius of the ellipse used to 
	 round off the corners of the rectangle.
	 */
	int getRoundingY() { return ry; }

	/** Sets the y-axis radius of the ellipse. */
	void setRoundingY(int ry) { this->ry = ry; }

	/** Sets priority of transition this rectangle represents. */
	void setPriority(int priority) { this->priority = priority; }

	/** Gets priority of transiotion. */
	int getPriority() { return priority; }	
	
	/** Sets time associated with transition this rectangle represents. */	
	void setTime(int time) { this->time = time; }	

	/** Gets time of transition. */
	int getTime() { return time; }		

	/** Sets whether this rectangle represents transition of petri net. */
	void setTransition(bool value) { transition = value; }
	
	/** Gets information whether this rectangle represents transition of petri net. */
	bool isTransition() { return transition; }
};

} // end of namespace SVGOutput

#endif // _SVG_RECTANGLE
