#ifndef _XML_TEXT_FORMATTER
#define _XML_TEXT_FORMATTER

#include <qtextstream.h>
#include "IXMLTagStream.h"

namespace SVGOutput
{

/**
 Base structure for stream manipulators
 */
class iomanip
{
public:
	/** Outputs data to QTextStrream. */
	virtual QTextStream &output(QTextStream &out) const = 0;
	virtual ~iomanip() { }
};

/**
 This manipulator writes out all attributes of given tag.
 */
class attributes : public iomanip
{
private:
	attributes_map &map;
	
public:	
	attributes(XMLTag *tag): map(tag->getAttributesMap()) { }
	~attributes() { }

	QTextStream &output(QTextStream &out) const
	{
		out << " ";

		attributes_map::iterator it = map.begin();
		while(it != map.end())
		{
			out << it->first << "=\"" << it->second << "\" ";
			it++;
		}

		return out;
	}

	void operator=(attributes &a) { map = a.map; }
};

/**
 This manipulator writes out name of given tag.
 */
class name : public iomanip
{
private:
	QString tagName;
	
public:	
	name(XMLTag *tag): tagName(tag->getName()) { }
	~name() { }

	QTextStream &output(QTextStream &out) const
	{
		out << tagName;

		return out;
	}
};

/**
 This manipulator writes out escaped name of given tag.
 It is used for tag of type ttText.
 */
class escaped_name : public iomanip
{
private:
	QString tagName;
	
public:	
	escaped_name(XMLTag *tag): tagName(tag->getName()) { }
	~escaped_name() { }

	QTextStream &output(QTextStream &out) const
	{
	    QString xml = tagName;
	    xml.replace("&", "&amp;");
	    xml.replace("<", "&lt;");
	    xml.replace(">", "&gt;");
	    xml.replace("\'", "&apos;");
	    xml.replace("\"", "&quot;");
	    
	    out << xml;
	    return out;
	}
};

/**
 This manipulator indents text.
 */
class indent : public iomanip
{
private: 
	int in;

public:	
	indent(int in): in(in) { }
	~indent() { }

	QTextStream &output(QTextStream &out) const
	{
		int _in = in;
		for(; _in; _in--)
			out << "\t";

		return out;
	}
};

/** Operator defined for text streams and manipulators. */
QTextStream &operator<<(QTextStream &out, const iomanip &io);

/**
 Provides means of writing object implmeneting 
 IXMLTagStream interface to a QTextStream.
 */
class XMLTextWriter
{
	IXMLTagStream &in;

public:
	/** 
	 Constructs new XMLTextWriter and associates 
	 it with IXMLTagStream.
	 */
	XMLTextWriter(IXMLTagStream &in): in(in) { }

	/** Writes tags to QTextStream. */
	void write(QTextStream &out);
};

} // end of namespace SVGOutput

#endif // _XML_TEXT_FORMATTER
