/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cellipse.h"

//--------------------------------------------------------------------------
/**
  Constructs an ellipse at position (\a x, \a y) and
  \a w by \a h, on \a canvas
  \param etag the ellipse tag,
  \param x the horizontal position of the ellipse,
  \param y the horizontal position of the ellipse,
  \param w the ellipse width,
  \param h the ellipse height,
  \param canvas the parent canvas.
*/

CEllipse::CEllipse (int etag, int x, int y, int w, int h, QCanvas * canvas):
QCanvasPolygonalItem (canvas), CElement(etag)
{
  setX (x);
  setY (y);
  ellipseWidth = w;
  ellipseHeight = h;
  dotsLabelFont = QFont("times",14);
}


//--------------------------------------------------------------------------
/**  Returns the points bounding the ellipse. */

QPointArray CEllipse::areaPoints () const
{
  QPointArray points;
  points =
    QPointArray (QRect (int(x()-4), int(y()-4), ellipseWidth+8, ellipseHeight+8));
  return points;
}


//--------------------------------------------------------------------------
/**  Sets the number of dots to \a d. If \a d < 0, the number of dots = 0 is set.
*/

void CEllipse::setDots (int d)
{
  if (d < 0)
    dots = 0;
  else
    dots = d;
  update();
}



//--------------------------------------------------------------------------
/**  Draw the shape. The \a pen and \a brush of \a the painter are already set
  to \a pen() and \a brush() (\a setPen() and \a setBrush() may be used. */

void CEllipse::drawShape(QPainter & painter)
{
  painter.drawEllipse(int(x()), int(y()), ellipseWidth, ellipseHeight);
  
  painter.setBrush (QBrush (QColor (0, 0, 0), Qt::SolidPattern));
  painter.setPen (QPen (Qt::black, 1));
  painter.setClipping(false);
  
  // drawing dots (tokens)
  switch (dots)
  {
    case 0:
      break;
    case 1:
      painter.drawEllipse((int)(x() + (width() - 6) / 2), 
                          (int)(y() + (height() - 6) / 2), 6, 6);
      break;
    case 2:
      painter.drawEllipse((int)(x() + (width() - 6) / 3),
                          (int)(y() + (height() - 6) / 2), 6, 6);
      painter.drawEllipse((int)(x() + 2 * (width() - 6) / 3),
                          (int)(y() + (height() - 6) / 2), 6, 6);
      break;
    case 3:
      painter.drawEllipse((int)(x() + (width() - 6) / 2),
                          (int)(y() + (height() - 6) / 3) - 1, 6, 6);
      painter.drawEllipse((int)(x() + (width() - 6) / 3),
                          (int)(y() + 2 * (height() - 6) / 3) + 1, 6, 6);
      painter.drawEllipse((int)(x() + 2 * (width() - 6) / 3),
                          (int)(y() + 2 * (height() - 6) / 3) + 1, 6, 6);
      break;
    case 4:
      painter.drawEllipse((int)(x() + (width() - 6) / 3),
                          (int)(y() + (height() - 6) / 3) - 1, 6, 6);
      painter.drawEllipse((int)(x() + 2 * (width() - 6) / 3),
                          (int)(y() + (height() - 6) / 3) - 1, 6, 6);
      painter.drawEllipse((int)(x() + (width() - 6) / 3),
                          (int)(y() + 2 * (height() - 6) / 3) + 1, 6, 6);
      painter.drawEllipse((int)(x() + 2 * (width() - 6) / 3),
                          (int)(y() + 2 * (height() - 6) / 3) + 1, 6, 6);
      break;
    default:
    {
      QString dotsLabel;
      dotsLabel = QString::number(dots);
      painter.setFont(dotsLabelFont);
      painter.drawText(boundingRect(),Qt::AlignHCenter | Qt::AlignVCenter, dotsLabel); 
    }
  }
  
  // if is selected some extra boxes are drawn
  if(isSelected())
  {
    painter.setBrush(QBrush(QColor(0, 0, 200), Qt::SolidPattern));
    painter.setPen(QPen (Qt::NoPen));
    painter.setClipping(false);
    
    painter.drawRect((int)x()-3, (int)y()-3, 6, 6); //left top
    painter.drawRect((int)(x()+width()/2)-3, (int)y()-3, 6, 6);
    painter.drawRect((int)(x()+width())-6+3, (int)y()-3, 6, 6);
    painter.drawRect((int)x()-3, (int)(y()+height()/2)-3, 6, 6);
    painter.drawRect((int)(x()+width())-6+3,(int)(y()+height()/2)-3, 6, 6);
    painter.drawRect((int)x()-3, (int)(y()+height())-6+3, 6, 6);
    painter.drawRect((int)(x()+width()/2)-3, (int)(y()+height())-6+3, 6, 6);
    painter.drawRect((int)(x()+width())-6+3, (int)(y()+height())-6+3, 6, 6);
    
    painter.setPen(QPen());
    painter.setBrush(QBrush());
  }
}


//--------------------------------------------------------------------------
/** Sets the color of the ellipse. */

void CEllipse::setColor (const QColor & c)
{
  QPen p = pen();
  p.setWidth(pen().width());
  p.setColor(c);
  setPen(p);
}














