/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "clabel.h"

//--------------------------------------------------------------------------
/**
  Constructs a label at position (\a x, \a y) with \a text on \a canvas.
  \param ltag the label tag,
  \param x the horizontal position of the label,
  \param y the horizontal position of the label,
  \param text the label text,
  \param yes if \a yes is \a true the label is placed inside a box,
  \param canvas the parent canvas.
*/

CLabel::CLabel(int ltag, int x, int y, QString text, bool yes, QCanvas* canvas):
    QCanvasRectangle(canvas)
{
  labelTag = ltag;
  setX(x);
  setY(y);
  labelText = text;
  labelFrame = yes;

  //  calculate the size of text
  QRect rect;
  QFontMetrics fontMetrics(labelFont);
  rect = fontMetrics.boundingRect(text);
  setSize(rect.width()+8, rect.height()+8);
  invalidate();  // invalidates all information about the area covered by the canvas item.
}


//--------------------------------------------------------------------------
/** Draw the shape. The \a pen and \a brush of \a the painter are already set
  to \a pen() and \a brush() (\a setPen() and \a setBrush() may be used. */
  
void CLabel::draw(QPainter & painter)
{
  // if labelFrame == true draw box
  if(labelFrame == true)
  {
    QCanvasRectangle::draw(painter);
  }

  // draw text
  painter.setFont(labelFont);
  painter.drawText(int(x()), int(y()), width(), height(),
  Qt::AlignCenter | Qt::WordBreak, labelText);

  // if is selected, some extra boxes are drawn
  if(isSelected())
  {
    painter.setBrush(QBrush(QColor(0,0,200), Qt::SolidPattern)); // blue
    painter.setPen (QPen(Qt::NoPen));
    painter.setClipping(false);

    painter.drawRect((int)x()-3, (int)y()-3, 6, 6); //left top
    painter.drawRect((int)(x()+width()/2)-3, (int)y()-3, 6, 6);
    painter.drawRect((int)(x()+width())-6+3, (int)y()-3, 6, 6);
    painter.drawRect((int)x()-3, (int)(y()+height()/2)-3, 6, 6);
    painter.drawRect((int)(x()+width())-6+3,(int)(y()+height()/2)-3, 6, 6);
    painter.drawRect((int)x()-3, (int)(y()+height())-6+3, 6, 6);
    painter.drawRect((int)(x()+width()/2)-3, (int)(y()+height())-6+3, 6, 6);
    painter.drawRect((int)(x()+width())-6+3, (int)(y()+height())-6+3, 6, 6);

    painter.setPen(QPen());
    painter.setBrush(QBrush());
  }
}


//--------------------------------------------------------------------------
/** Sets the text of the label. */

void CLabel::setText(QString t)
{
  labelText = t;

  //  calculate the new size of label
  QRect rect;
  QFontMetrics fontMetrics(labelFont);
  rect = fontMetrics.boundingRect(labelText);
  setSize(rect.width()+8, rect.height()+8);
  update();
  canvas()->update();
}


//--------------------------------------------------------------------------
/** Sets the font fot dots label. */

void CLabel::setFont(const QFont & f)
{
  labelFont = f;

  //  calculate the new size of label
  QRect rect;
  QFontMetrics fontMetrics(labelFont);
  rect = fontMetrics.boundingRect(labelText);
  setSize(rect.width()+8, rect.height()+2);
  update();
}


//--------------------------------------------------------------------------
/** Sets the frame attribut value to \a yes.
  If \a yes = \a true, the label is placed inside a box. */
  
void CLabel::setFrame(bool yes)
{
  labelFrame = yes;
  update();
}


//--------------------------------------------------------------------------
/** Sets the color of the label. */

void CLabel::setColor(const QColor & c)
{
  QPen p = pen();
  p.setWidth(pen().width());
  p.setColor(c);
  setPen(p);
}


//--------------------------------------------------------------------------
/** Sets the label tag to \a ltag. */

void CLabel::setTag(int ltag)
{
  labelTag = ltag;
}



//--------------------------------------------------------------------------
/** Returns the points bounding the rectangle. */

QPointArray CLabel::areaPoints () const
{
  QPointArray points;
  points =
    QPointArray(QRect(int(x()-3), int(y()-3), int(width()+6), int(height()+6)));
  return points;
}




